@@ -1,30 +1,29 @@
-/*****************************************************************************
- * libVLC backend for the Phonon library                                     *
- *                                                                           *
- * Copyright (C) 2007-2008 Tanguy Krotoff <tkrotoff@gmail.com>               *
- * Copyright (C) 2008 Lukas Durfina <lukas.durfina@gmail.com>                *
- * Copyright (C) 2009 Fathi Boudra <fabo@kde.org>                            *
- * Copyright (C) 2009-2010 vlc-phonon AUTHORS                                *
- *                                                                           *
- * This program is free software; you can redistribute it and/or             *
- * modify it under the terms of the GNU Lesser General Public                *
- * License as published by the Free Software Foundation; either              *
- * version 2.1 of the License, or (at your option) any later version.        *
- *                                                                           *
- * This program is distributed in the hope that it will be useful,           *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         *
- * Lesser General Public License for more details.                           *
- *                                                                           *
- * You should have received a copy of the GNU Lesser General Public          *
- * License along with this package; if not, write to the Free Software       *
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA *
- *****************************************************************************/
+/*
+    Copyright (C) 2007-2008 Tanguy Krotoff <tkrotoff@gmail.com>
+    Copyright (C) 2008 Lukas Durfina <lukas.durfina@gmail.com>
+    Copyright (C) 2009 Fathi Boudra <fabo@kde.org>
+    Copyright (C) 2009-2011 vlc-phonon AUTHORS <kde-multimedia@kde.org>
+
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #include "mediacontroller.h"
 
 #include <phonon/GlobalDescriptionContainer>
 
+#include <QTimer>
+
 #include "utils/debug.h"
 #include "utils/libvlc.h"
 #include "mediaplayer.h"
@@ -37,7 +36,12 @@ namespace VLC {
 #endif
 
 MediaController::MediaController()
-    : m_player(0)
+    : m_subtitleAutodetect(true)
+    , m_subtitleEncoding("UTF-8")
+    , m_subtitleFontChanged(false)
+    , m_player(0)
+    , m_refreshTimer(new QTimer(dynamic_cast<QObject *>(this)))
+    , m_attemptingAutoplay(false)
 {
     GlobalSubtitles::instance()->register_(this);
     GlobalAudioChannels::instance()->register_(this);
@@ -94,8 +98,6 @@ QVariant MediaController::interfaceCall(Interface iface, int i_command, const QL
             }
             setCurrentChapter(arguments.first().toInt());
             return true;
-        default:
-            error() << Q_FUNC_INFO << "unsupported AddonInterface::ChapterInterface command:" << i_command;
         }
         break;
     case AddonInterface::TitleInterface:
@@ -120,8 +122,6 @@ QVariant MediaController::interfaceCall(Interface iface, int i_command, const QL
             }
             setAutoplayTitles(arguments.first().toBool());
             return true;
-        default:
-            error() << Q_FUNC_INFO << "unsupported AddonInterface::TitleInterface command:" << i_command;
         }
         break;
     case AddonInterface::AngleInterface:
@@ -140,8 +140,39 @@ QVariant MediaController::interfaceCall(Interface iface, int i_command, const QL
             }
             setCurrentSubtitle(arguments.first().value<SubtitleDescription>());
             return true;
-        default:
-            error() << Q_FUNC_INFO << "unsupported AddonInterface::SubtitleInterface command:" << i_command;
+        case AddonInterface::setCurrentSubtitleFile:
+            if (arguments.isEmpty() || !arguments.first().canConvert<QUrl>()) {
+                error() << Q_FUNC_INFO << " arguments invalid";
+                return false;
+            }
+            setCurrentSubtitleFile(arguments.first().value<QUrl>());
+        case AddonInterface::subtitleAutodetect:
+            return QVariant::fromValue(subtitleAutodetect());
+        case AddonInterface::setSubtitleAutodetect:
+            if (arguments.isEmpty() || !arguments.first().canConvert<bool>()) {
+                error() << Q_FUNC_INFO << " arguments invalid";
+                return false;
+            }
+            setSubtitleAutodetect(arguments.first().value<bool>());
+            return true;
+        case AddonInterface::subtitleEncoding:
+            return subtitleEncoding();
+        case AddonInterface::setSubtitleEncoding:
+            if (arguments.isEmpty() || !arguments.first().canConvert<QString>()) {
+                error() << Q_FUNC_INFO << " arguments invalid";
+                return false;
+            }
+            setSubtitleEncoding(arguments.first().value<QString>());
+            return true;
+        case AddonInterface::subtitleFont:
+            return subtitleFont();
+        case AddonInterface::setSubtitleFont:
+            if (arguments.isEmpty() || !arguments.first().canConvert<QFont>()) {
+                error() << Q_FUNC_INFO << " arguments invalid";
+                return false;
+            }
+            setSubtitleFont(arguments.first().value<QFont>());
+            return true;
         }
         break;
     case AddonInterface::AudioChannelInterface:
@@ -157,14 +188,12 @@ QVariant MediaController::interfaceCall(Interface iface, int i_command, const QL
             }
             setCurrentAudioChannel(arguments.first().value<AudioChannelDescription>());
             return true;
-        default:
-            error() << Q_FUNC_INFO << "unsupported AddonInterface::AudioChannelInterface command:" << i_command;
         }
         break;
-    default:
-        error() << Q_FUNC_INFO << "unsupported AddonInterface::Interface:" << iface;
     }
 
+    error() << Q_FUNC_INFO << "unsupported AddonInterface::Interface:" << iface;
+
     return QVariant();
 }
 
@@ -188,10 +217,10 @@ void MediaController::resetMembers()
     m_currentChapter = 0;
     m_availableChapters = 0;
 
-    m_currentTitle = 0;
+    m_currentTitle = 1;
     m_availableTitles = 0;
 
-    m_autoPlayTitles = false;
+    m_attemptingAutoplay = false;
 }
 
 // ----------------------------- Audio Channel ------------------------------ //
@@ -245,8 +274,11 @@ void MediaController::refreshAudioChannels()
 // -------------------------------- Subtitle -------------------------------- //
 void MediaController::setCurrentSubtitle(const Phonon::SubtitleDescription &subtitle)
 {
+    DEBUG_BLOCK;
     QString type = subtitle.property("type").toString();
 
+    debug() << subtitle;
+
     if (type == "file") {
         QString filename = subtitle.property("name").toString();
         if (!filename.isEmpty()) {
@@ -261,6 +293,7 @@ void MediaController::setCurrentSubtitle(const Phonon::SubtitleDescription &subt
         }
     } else {
         const int localIndex = GlobalSubtitles::instance()->localIdFor(this, subtitle.index());
+        debug () << "localid" << localIndex;
         if (!m_player->setSubtitle(localIndex))
             error() << "libVLC:" << LibVLC::errorMessage();
         else
@@ -268,6 +301,24 @@ void MediaController::setCurrentSubtitle(const Phonon::SubtitleDescription &subt
     }
 }
 
+void MediaController::setCurrentSubtitleFile(const QUrl &url)
+{
+    const QString file = url.toLocalFile();
+    if (!m_player->setSubtitle(file))
+        error() << "libVLC failed to set subtitle file:" << LibVLC::errorMessage();
+    // Unfortunately the addition of SPUs does not trigger an event in the
+    // VLC mediaplayer, yet the actual addition to the descriptor is async.
+    // So for the time being our best shot at getting an up-to-date list of SPUs
+    // is shooting in the dark and hoping we hit something.
+    // Refresha after 1, 2 and 5 seconds. If we have no updated list after 5
+    // seconds we are out of luck.
+    // https://trac.videolan.org/vlc/ticket/9796
+    QObject *mediaObject = dynamic_cast<QObject *>(this); // MediaObject : QObject, MediaController
+    m_refreshTimer->singleShot(1 * 1000, mediaObject, SLOT(refreshDescriptors()));
+    m_refreshTimer->singleShot(2 * 1000, mediaObject, SLOT(refreshDescriptors()));
+    m_refreshTimer->singleShot(5 * 1000, mediaObject, SLOT(refreshDescriptors()));
+}
+
 QList<Phonon::SubtitleDescription> MediaController::availableSubtitles() const
 {
     return GlobalSubtitles::instance()->listFor(this);
@@ -285,12 +336,10 @@ void MediaController::refreshSubtitles()
 
     const int currentSubtitleId = m_player->subtitle();
 
-    int idCount = 0;
     VLC_FOREACH_TRACK(it, m_player->videoSubtitleDescription()) {
-        // LibVLC's internal ID is broken, so we simply count up as internally
-        // the setter will simply go by position in list anyway.
-        GlobalSubtitles::instance()->add(this, idCount, QString::fromUtf8(it->psz_name), "");
-        if (idCount == currentSubtitleId) {
+        debug() << "found subtitle" << it->psz_name << "[" << it->i_id << "]";
+        GlobalSubtitles::instance()->add(this, it->i_id, QString::fromUtf8(it->psz_name), "");
+        if (it->i_id == currentSubtitleId) {
 #ifdef __GNUC__
 #warning GlobalDescriptionContainer does not allow reverse resolution from local to descriptor!
 #endif
@@ -301,13 +350,42 @@ void MediaController::refreshSubtitles()
                 }
             }
         }
-
-        ++idCount;
     }
 
     emit availableSubtitlesChanged();
 }
 
+bool MediaController::subtitleAutodetect() const
+{
+    return m_subtitleAutodetect;
+}
+
+void MediaController::setSubtitleAutodetect(bool enabled)
+{
+    m_subtitleAutodetect = enabled;
+}
+
+QString MediaController::subtitleEncoding() const
+{
+    return m_subtitleEncoding;
+}
+
+void MediaController::setSubtitleEncoding(const QString &encoding)
+{
+    m_subtitleEncoding = encoding;
+}
+
+QFont MediaController::subtitleFont() const
+{
+    return m_subtitleFont;
+}
+
+void MediaController::setSubtitleFont(const QFont &font)
+{
+    m_subtitleFontChanged = true;
+    m_subtitleFont = font;
+}
+
 // --------------------------------- Title ---------------------------------- //
 void MediaController::setCurrentTitle(int title)
 {
@@ -316,18 +394,13 @@ void MediaController::setCurrentTitle(int title)
 
     switch (source().discType()) {
     case Cd:
-#ifdef __GNUC__
-#warning use media subitem to set track of audiocd
-#endif
-        // Leave for MediaObject to handle.
+        m_player->setCdTrack(title);
         return;
     case Dvd:
     case Vcd:
-#if (PHONON_VERSION >= PHONON_VERSION_CHECK(4, 7, 0))
     case BluRay:
         m_player->setTitle(title);
         return;
-#endif
     case NoDisc:
         warning() << "Current media source is not a CD, DVD or VCD!";
         return;
@@ -361,7 +434,7 @@ void MediaController::refreshTitles()
 {
     m_availableTitles = 0;
 
-    VLC_FOREACH_TRACK(it, m_player->videoTitleDescription()) {
+    VLC_FOREACH_TRACK(it, m_player->titleDescription()) {
         ++m_availableTitles;
         emit availableTitlesChanged(m_availableTitles);
     }
